@@ -13,38 +13,43 @@
 
 #include <QCheckBox>
 #include <QDialogButtonBox>
+#include <QGridLayout>
 #include <QGroupBox>
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenu>
 #include <QPushButton>
 #include <QRegularExpression>
-#include <QGridLayout>
 
-#include <KLocalizedString>
-#include <KMessageBox>
 #include <KGuiItem>
 #include <KHistoryComboBox>
+#include <KLocalizedString>
+#include <KMessageBox>
 
 #include <assert.h>
 
 KFindDialog::KFindDialog(QWidget *parent, long options, const QStringList &findStrings, bool hasSelection, bool replaceDialog)
-    : QDialog(parent),
-      d(new KFindDialogPrivate(this))
+    : KFindDialog(*new KFindDialogPrivate(this), parent, options, findStrings, hasSelection, replaceDialog)
 {
     setWindowTitle(i18n("Find Text"));
+}
+
+KFindDialog::KFindDialog(KFindDialogPrivate &dd, QWidget *parent, long options, const QStringList &findStrings, bool hasSelection, bool replaceDialog)
+    : QDialog(parent)
+    , d(&dd)
+{
+    Q_D(KFindDialog);
 
     d->init(replaceDialog, findStrings, hasSelection);
     setOptions(options);
 }
 
-KFindDialog::~KFindDialog()
-{
-    delete d;
-}
+KFindDialog::~KFindDialog() = default;
 
 QWidget *KFindDialog::findExtension() const
 {
+    Q_D(const KFindDialog);
+
     if (!d->findExtension) {
         d->findExtension = new QWidget(d->findGrp);
         d->findLayout->addWidget(d->findExtension, 3, 0, 1, 2);
@@ -55,11 +60,15 @@ QWidget *KFindDialog::findExtension() const
 
 QStringList KFindDialog::findHistory() const
 {
+    Q_D(const KFindDialog);
+
     return d->find->historyItems();
 }
 
-void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings, bool hasSelection)
+void KFindDialogPrivate::init(bool forReplace, const QStringList &_findStrings, bool hasSelection)
 {
+    Q_Q(KFindDialog);
+
     // Create common parts of dialog.
     QVBoxLayout *topLayout = new QVBoxLayout(q);
 
@@ -153,7 +162,7 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
     // tab order
     q->setTabOrder(find, regExp);
     q->setTabOrder(regExp, regExpItem);
-    q->setTabOrder(regExpItem, replace); //findExtension widgets are inserted in showEvent()
+    q->setTabOrder(regExpItem, replace); // findExtension widgets are inserted in showEvent()
     q->setTabOrder(replace, backRef);
     q->setTabOrder(backRef, backRefItem);
     q->setTabOrder(backRefItem, caseSensitive);
@@ -178,64 +187,57 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
     buttonOk->setEnabled(!q->pattern().isEmpty());
 
     if (forReplace) {
-        KGuiItem::assign(buttonOk, KGuiItem(i18n("&Replace"), QString(),
-                                            i18n("Start replace"),
-                                            i18n("<qt>If you press the <b>Replace</b> button, the text you entered "
-                                                    "above is searched for within the document and any occurrence is "
-                                                    "replaced with the replacement text.</qt>")));
+        KGuiItem::assign(buttonOk,
+                         KGuiItem(i18n("&Replace"),
+                                  QString(),
+                                  i18n("Start replace"),
+                                  i18n("<qt>If you press the <b>Replace</b> button, the text you entered "
+                                       "above is searched for within the document and any occurrence is "
+                                       "replaced with the replacement text.</qt>")));
     } else {
-        KGuiItem::assign(buttonOk, KGuiItem(i18n("&Find"), QStringLiteral("edit-find"),
-                                            i18n("Start searching"),
-                                            i18n("<qt>If you press the <b>Find</b> button, the text you entered "
-                                                    "above is searched for within the document.</qt>")));
+        KGuiItem::assign(buttonOk,
+                         KGuiItem(i18n("&Find"),
+                                  QStringLiteral("edit-find"),
+                                  i18n("Start searching"),
+                                  i18n("<qt>If you press the <b>Find</b> button, the text you entered "
+                                       "above is searched for within the document.</qt>")));
     }
 
     // QWhatsthis texts
-    find->setWhatsThis(i18n(
-                           "Enter a pattern to search for, or select a previous pattern from "
-                           "the list."));
-    regExp->setWhatsThis(i18n(
-                             "If enabled, search for a regular expression."));
-    regExpItem->setWhatsThis(i18n(
-                                 "Click here to edit your regular expression using a graphical editor."));
-    replace->setWhatsThis(i18n(
-                              "Enter a replacement string, or select a previous one from the list."));
-    backRef->setWhatsThis(i18n(
-                              "<qt>If enabled, any occurrence of <code><b>\\N</b></code>, where "
-                              "<code><b>N</b></code> is an integer number, will be replaced with "
-                              "the corresponding capture (\"parenthesized substring\") from the "
-                              "pattern.<p>To include (a literal <code><b>\\N</b></code> in your "
-                              "replacement, put an extra backslash in front of it, like "
-                              "<code><b>\\\\N</b></code>.</p></qt>"));
-    backRefItem->setWhatsThis(i18n(
-                                  "Click for a menu of available captures."));
-    wholeWordsOnly->setWhatsThis(i18n(
-                                     "Require word boundaries in both ends of a match to succeed."));
-    fromCursor->setWhatsThis(i18n(
-                                 "Start searching at the current cursor location rather than at the top."));
-    selectedText->setWhatsThis(i18n(
-                                   "Only search within the current selection."));
-    caseSensitive->setWhatsThis(i18n(
-                                    "Perform a case sensitive search: entering the pattern "
-                                    "'Joe' will not match 'joe' or 'JOE', only 'Joe'."));
-    findBackwards->setWhatsThis(i18n(
-                                    "Search backwards."));
-    promptOnReplace->setWhatsThis(i18n(
-                                      "Ask before replacing each match found."));
+    find->setWhatsThis(i18n("Enter a pattern to search for, or select a previous pattern from the list."));
+    regExp->setWhatsThis(i18n("If enabled, search for a regular expression."));
+    regExpItem->setWhatsThis(i18n("Click here to edit your regular expression using a graphical editor."));
+    replace->setWhatsThis(i18n("Enter a replacement string, or select a previous one from the list."));
+    backRef->setWhatsThis(
+        i18n("<qt>If enabled, any occurrence of <code><b>\\N</b></code>, where "
+             "<code><b>N</b></code> is an integer number, will be replaced with "
+             "the corresponding capture (\"parenthesized substring\") from the "
+             "pattern.<p>To include (a literal <code><b>\\N</b></code> in your "
+             "replacement, put an extra backslash in front of it, like "
+             "<code><b>\\\\N</b></code>.</p></qt>"));
+    backRefItem->setWhatsThis(i18n("Click for a menu of available captures."));
+    wholeWordsOnly->setWhatsThis(i18n("Require word boundaries in both ends of a match to succeed."));
+    fromCursor->setWhatsThis(i18n("Start searching at the current cursor location rather than at the top."));
+    selectedText->setWhatsThis(i18n("Only search within the current selection."));
+    caseSensitive->setWhatsThis(i18n("Perform a case sensitive search: entering the pattern 'Joe' will not match 'joe' or 'JOE', only 'Joe'."));
+    findBackwards->setWhatsThis(i18n("Search backwards."));
+    promptOnReplace->setWhatsThis(i18n("Ask before replacing each match found."));
 
     _k_textSearchChanged(find->lineEdit()->text());
 }
 
-void KFindDialog::KFindDialogPrivate::_k_textSearchChanged(const QString &text)
+void KFindDialogPrivate::_k_textSearchChanged(const QString &text)
 {
     buttonBox->button(QDialogButtonBox::Ok)->setEnabled(!text.isEmpty());
 }
 
 void KFindDialog::showEvent(QShowEvent *e)
 {
+    Q_D(KFindDialog);
+
     if (!d->initialShowDone) {
         d->initialShowDone = true; // only once
-        //qDebug() << "showEvent\n";
+        // qDebug() << "showEvent\n";
         if (!d->findStrings.isEmpty()) {
             setFindHistory(d->findStrings);
         }
@@ -245,7 +247,7 @@ void KFindDialog::showEvent(QShowEvent *e)
             d->find->lineEdit()->selectAll();
             d->pattern.clear();
         }
-        //maintain a user-friendly tab order
+        // maintain a user-friendly tab order
         if (d->findExtension) {
             QWidget *prev = d->regExpItem;
             const auto children = d->findExtension->findChildren<QWidget *>();
@@ -262,6 +264,8 @@ void KFindDialog::showEvent(QShowEvent *e)
 
 long KFindDialog::options() const
 {
+    Q_D(const KFindDialog);
+
     long options = 0;
 
     if (d->caseSensitive->isChecked()) {
@@ -287,19 +291,25 @@ long KFindDialog::options() const
 
 QString KFindDialog::pattern() const
 {
+    Q_D(const KFindDialog);
+
     return d->find->currentText();
 }
 
 void KFindDialog::setPattern(const QString &pattern)
 {
+    Q_D(KFindDialog);
+
     d->find->lineEdit()->setText(pattern);
     d->find->lineEdit()->selectAll();
     d->pattern = pattern;
-    //qDebug() << "setPattern " << pattern;
+    // qDebug() << "setPattern " << pattern;
 }
 
 void KFindDialog::setFindHistory(const QStringList &strings)
 {
+    Q_D(KFindDialog);
+
     if (!strings.isEmpty()) {
         d->find->setHistoryItems(strings, true);
         d->find->lineEdit()->setText(strings.first());
@@ -311,6 +321,8 @@ void KFindDialog::setFindHistory(const QStringList &strings)
 
 void KFindDialog::setHasSelection(bool hasSelection)
 {
+    Q_D(KFindDialog);
+
     if (hasSelection) {
         d->enabled |= KFind::SelectedText;
     } else {
@@ -323,7 +335,7 @@ void KFindDialog::setHasSelection(bool hasSelection)
     }
 }
 
-void KFindDialog::KFindDialogPrivate::_k_slotSelectedTextToggled(bool selec)
+void KFindDialogPrivate::_k_slotSelectedTextToggled(bool selec)
 {
     // From cursor doesn't make sense if we have a selection
     fromCursor->setEnabled(!selec && (enabled & KFind::FromCursor));
@@ -334,6 +346,8 @@ void KFindDialog::KFindDialogPrivate::_k_slotSelectedTextToggled(bool selec)
 
 void KFindDialog::setHasCursor(bool hasCursor)
 {
+    Q_D(KFindDialog);
+
     if (hasCursor) {
         d->enabled |= KFind::FromCursor;
     } else {
@@ -345,6 +359,8 @@ void KFindDialog::setHasCursor(bool hasCursor)
 
 void KFindDialog::setSupportsBackwardsFind(bool supports)
 {
+    Q_D(KFindDialog);
+
     // ########## Shouldn't this hide the checkbox instead?
     if (supports) {
         d->enabled |= KFind::FindBackwards;
@@ -357,6 +373,8 @@ void KFindDialog::setSupportsBackwardsFind(bool supports)
 
 void KFindDialog::setSupportsCaseSensitiveFind(bool supports)
 {
+    Q_D(KFindDialog);
+
     // ########## This should hide the checkbox instead
     if (supports) {
         d->enabled |= KFind::CaseSensitive;
@@ -369,6 +387,8 @@ void KFindDialog::setSupportsCaseSensitiveFind(bool supports)
 
 void KFindDialog::setSupportsWholeWordsFind(bool supports)
 {
+    Q_D(KFindDialog);
+
     // ########## This should hide the checkbox instead
     if (supports) {
         d->enabled |= KFind::WholeWordsOnly;
@@ -381,6 +401,8 @@ void KFindDialog::setSupportsWholeWordsFind(bool supports)
 
 void KFindDialog::setSupportsRegularExpressionFind(bool supports)
 {
+    Q_D(KFindDialog);
+
     if (supports) {
         d->enabled |= KFind::RegularExpression;
     } else {
@@ -399,6 +421,8 @@ void KFindDialog::setSupportsRegularExpressionFind(bool supports)
 
 void KFindDialog::setOptions(long options)
 {
+    Q_D(KFindDialog);
+
     d->caseSensitive->setChecked((d->enabled & KFind::CaseSensitive) && (options & KFind::CaseSensitive));
     d->wholeWordsOnly->setChecked((d->enabled & KFind::WholeWordsOnly) && (options & KFind::WholeWordsOnly));
     d->fromCursor->setChecked((d->enabled & KFind::FromCursor) && (options & KFind::FromCursor));
@@ -409,34 +433,39 @@ void KFindDialog::setOptions(long options)
 
 // Create a popup menu with a list of regular expression terms, to help the user
 // compose a regular expression search pattern.
-void KFindDialog::KFindDialogPrivate::_k_showPatterns()
+void KFindDialogPrivate::_k_showPatterns()
 {
+    Q_Q(KFindDialog);
+
     typedef struct {
         const char *description;
         const char *regExp;
         int cursorAdjustment;
     } Term;
     static const Term items[] = {
-        { I18N_NOOP("Any Character"),                 ".",        0 },
-        { I18N_NOOP("Start of Line"),                 "^",        0 },
-        { I18N_NOOP("End of Line"),                   "$",        0 },
-        { I18N_NOOP("Set of Characters"),             "[]",       -1 },
-        { I18N_NOOP("Repeats, Zero or More Times"),   "*",        0 },
-        { I18N_NOOP("Repeats, One or More Times"),    "+",        0 },
-        { I18N_NOOP("Optional"),                      "?",        0 },
-        { I18N_NOOP("Escape"),                        "\\",       0 },
-        { I18N_NOOP("TAB"),                           "\\t",      0 },
-        { I18N_NOOP("Newline"),                       "\\n",      0 },
-        { I18N_NOOP("Carriage Return"),               "\\r",      0 },
-        { I18N_NOOP("White Space"),                   "\\s",      0 },
-        { I18N_NOOP("Digit"),                         "\\d",      0 },
+        {I18N_NOOP("Any Character"), ".", 0},
+        {I18N_NOOP("Start of Line"), "^", 0},
+        {I18N_NOOP("End of Line"), "$", 0},
+        {I18N_NOOP("Set of Characters"), "[]", -1},
+        {I18N_NOOP("Repeats, Zero or More Times"), "*", 0},
+        {I18N_NOOP("Repeats, One or More Times"), "+", 0},
+        {I18N_NOOP("Optional"), "?", 0},
+        {I18N_NOOP("Escape"), "\\", 0},
+        {I18N_NOOP("TAB"), "\\t", 0},
+        {I18N_NOOP("Newline"), "\\n", 0},
+        {I18N_NOOP("Carriage Return"), "\\r", 0},
+        {I18N_NOOP("White Space"), "\\s", 0},
+        {I18N_NOOP("Digit"), "\\d", 0},
     };
 
     class RegExpAction : public QAction
     {
     public:
         RegExpAction(QObject *parent, const QString &text, const QString &regExp, int cursor)
-            : QAction(text, parent), mText(text), mRegExp(regExp), mCursor(cursor)
+            : QAction(text, parent)
+            , mText(text)
+            , mRegExp(regExp)
+            , mCursor(cursor)
         {
         }
 
@@ -459,14 +488,11 @@ void KFindDialog::KFindDialogPrivate::_k_showPatterns()
         int mCursor;
     };
 
-
     // Populate the popup menu.
     if (!patterns) {
         patterns = new QMenu(q);
         for (const Term &item : items) {
-            patterns->addAction(new RegExpAction(patterns, i18n(item.description),
-                                                    QLatin1String(item.regExp),
-                                                    item.cursorAdjustment));
+            patterns->addAction(new RegExpAction(patterns, i18n(item.description), QLatin1String(item.regExp), item.cursorAdjustment));
         }
     }
 
@@ -487,7 +513,9 @@ class PlaceHolderAction : public QAction
 {
 public:
     PlaceHolderAction(QObject *parent, const QString &text, int id)
-        : QAction(text, parent), mText(text), mId(id)
+        : QAction(text, parent)
+        , mText(text)
+        , mId(id)
     {
     }
 
@@ -507,8 +535,10 @@ private:
 
 // Create a popup menu with a list of backreference terms, to help the user
 // compose a regular expression replacement pattern.
-void KFindDialog::KFindDialogPrivate::_k_showPlaceholders()
+void KFindDialogPrivate::_k_showPlaceholders()
 {
+    Q_Q(KFindDialog);
+
     // Populate the popup menu.
     if (!placeholders) {
         placeholders = new QMenu(q);
@@ -526,19 +556,23 @@ void KFindDialog::KFindDialogPrivate::_k_showPlaceholders()
     }
 }
 
-void KFindDialog::KFindDialogPrivate::_k_slotPlaceholdersAboutToShow()
+void KFindDialogPrivate::_k_slotPlaceholdersAboutToShow()
 {
+    Q_Q(KFindDialog);
+
     placeholders->clear();
     placeholders->addAction(new PlaceHolderAction(placeholders, i18n("Complete Match"), 0));
 
     const int n = QRegularExpression(q->pattern()).captureCount();
     for (int i = 1; i <= n; ++i) {
-        placeholders->addAction(new PlaceHolderAction(placeholders, i18n("Captured Text (%1)",  i), i));
+        placeholders->addAction(new PlaceHolderAction(placeholders, i18n("Captured Text (%1)", i), i));
     }
 }
 
-void KFindDialog::KFindDialogPrivate::_k_slotOk()
+void KFindDialogPrivate::_k_slotOk()
 {
+    Q_Q(KFindDialog);
+
     // Nothing to find?
     if (q->pattern().isEmpty()) {
         KMessageBox::error(q, i18n("You must enter some text to search for."));
@@ -558,12 +592,14 @@ void KFindDialog::KFindDialogPrivate::_k_slotOk()
     if (q->windowModality() != Qt::NonModal) {
         q->accept();
     }
-    emit q->okClicked();
+    Q_EMIT q->okClicked();
 }
 
-void KFindDialog::KFindDialogPrivate::_k_slotReject()
+void KFindDialogPrivate::_k_slotReject()
 {
-    emit q->cancelClicked();
+    Q_Q(KFindDialog);
+
+    Q_EMIT q->cancelClicked();
     q->reject();
 }
 
